import time
from calendar import monthrange

from netforce.model import Model,fields,get_model
from netforce.access import get_active_company, set_active_user, get_active_user, set_active_company

class ReportLaborCostSummary(Model):
    _name="clinic.report.labor.cost.summary"
    _string="Report Labor Cost Summary"
    _transient=True
    
    _fields={
        "date": fields.Date("Month"),
        "date_from": fields.Date("From", required=True),
        "date_to": fields.Date("To", required=True),
        'staff_id': fields.Many2One("clinic.staff","Staff"),
        "staff_type": fields.Selection([["doctor","Doctor"],["nurse","Nurse"],["staff","Staff"]],"Type"),
        'department_id': fields.Many2One("clinic.department","Department"),
        'branch_id': fields.Many2One("clinic.branch","Branch"),
        'level_id': fields.Many2One("clinic.staff.level","Level"),
        'cycle_id': fields.Many2One("clinic.cycle","Cycle"),
        'categ_id': fields.Many2One("clinic.staff.categ","Category"),
        'only_value': fields.Boolean("Only Amount"),
    }

    def _get_date_from(self,context={}):
        year,month=time.strftime("%Y-%m").split("-")
        return '%s-%s-01'%(year,month)

    def _get_date_to(self,context={}):
        year,month,day=time.strftime("%Y-%m-%d").split("-")
        weekday, total_day=monthrange(int(year), int(month))
        return "%s-%s-%s"%(year,month,total_day)

    def default_get(self,field_names=None,context={},**kw):
        defaults=context.get("defaults",{})
        date=defaults.get("date", time.strftime("%Y-%m-%d"))
        date_from=defaults.get("date_from", self._get_date_from())
        date_to=defaults.get("date_to", self._get_date_to())
        branch_id=defaults.get("branch_id")
        categ_id=defaults.get("categ_id")
        cycle_id=defaults.get("cycle_id")
        if cycle_id:
            cycle_id=int(cycle_id)
        if categ_id:
            categ_id=int(categ_id)
        if branch_id:
            branch_id=int(branch_id)
        department_id=defaults.get("department_id")
        if department_id:
            department_id=int(department_id)
        staff_id=defaults.get('staff_id')
        if staff_id:
            staff_id=int(staff_id)
        staff_type=defaults.get("staff_type","doctor")
        level_id=defaults.get("level_id")
        if level_id:
            level_id=int(level_id)
        res={
            'date': date,
            'date_from': date_from,
            'date_to': date_to,
            'staff_type': staff_type,
            'staff_id': staff_id and staff_id or None,
            'department_id': department_id or None,
            'branch_id': branch_id or None,
            'only_value': True,
            'categ_id': categ_id or None,
            'level_id': level_id or None,
            'cycle_id': cycle_id or None,
        }
        print('res ', res)
        return res
    
    def get_report_data(self,ids,context={}):
        user_id=get_active_user()
        company_id=get_active_company()

        set_active_user(1)
        set_active_company(1)

        defaults=self.default_get(context=context)
        date_from=defaults['date_from']
        date_to=defaults['date_to']
        staff_id=defaults['staff_id']
        staff_type=defaults['staff_type']
        only_value=defaults['only_value']
        department_id=defaults['department_id']
        branch_id=defaults['branch_id']
        categ_id=defaults['categ_id']
        level_id=defaults['level_id']
        cycle_id=defaults['cycle_id']
        dom=[]
        if ids:
            obj=self.browse(ids)[0]
            date_from=obj.date_from
            date_to=obj.date_to
            staff_id=obj.staff_id.id
            staff_type=obj.staff_type or "doctor"
            level_id=obj.level_id.id
            categ_id=obj.categ_id.id
            branch_id=obj.branch_id.id
            department_id=obj.department_id.id
            cycle_id=obj.cycle_id.id
            only_value=obj.only_value
        dom.append(['date','>=',date_from])
        dom.append(['date','<=',date_to])
        if staff_id:
            dom.append(['staff_id','=',staff_id])
        if staff_type:
            dom.append(['type','=',staff_type])
        if only_value:
            dom.append(['amount','!=',0])
        if categ_id:
            dom.append(['staff_id.categ_id','=',categ_id])
        if branch_id:
            dom.append(['labor_cost_id.cycle_item_id.branch_id','=',branch_id])
        if department_id:
            dom.append(['labor_cost_id.cycle_item_id.department_id','=',department_id])
        if cycle_id:
            dom.append(['labor_cost_id.cycle_item_id.cycle_id','=',cycle_id])
        staffs={}
        citems={}

        def int2ths(r):
            if type(r)==type(''):
                return r
            return "{0:,.0f}".format(r) 
        total_hdcase=0
        print("----> ", dom)
        for line in get_model("clinic.labor.cost.line").search_browse(dom):
            lcost=line.labor_cost_id
            citem=lcost.cycle_item_id
            dpt=citem.department_id
            qty=line.qty or 0
            total_hdcase+=qty
            amt=line.amount or 0
            staff=line.staff_id
            categ_name=''
            categ_id=None
            categ=staff.categ_id
            level_name=''
            if not staff:
                print('no staff', qty, amt)
                continue
            if level_id and staff.level_id.id !=level_id:
                continue
            if categ:
                categ_id=categ.id
                categ_name=categ.name or ""
            if staff.level_id:
                level=get_model("clinic.staff.level").browse(staff.level_id.id)
                level_name=level.name or ""
            staff_name='%s %s'%(staff.first_name, staff.last_name)
            if not staffs.get(staff_name):
                staffs[staff_name]={
                'number': staff.number or '',
                'first_name': staff.first_name or "",
                'staff_id': staff.id,
                'staff_type': staff.type,
                'staff_level': level_name,
                'categ_name': categ_name,
                'categ_id': categ_id,
                dpt.name: {
                    'amt': 0,
                    'qty': qty,
                    },
                }
            if not staffs[staff_name].get(dpt.name):
                staffs[staff_name].update({
                        dpt.name: {
                            'amt': 0,
                            'qty': qty,
                        }})
            staffs[staff_name][dpt.name]['amt']+=amt
            staffs[staff_name][dpt.name]['qty']+=qty

            if not citems.get(citem.id):
                qty=0
                for hdcase in citem.hd_cases:
                    if hdcase.state in ('waiting_payment', 'paid'):
                        qty+=1
                citems[citem.id]=qty
        lines=[]
        dom=[]
        if branch_id:
            dom.append(['branch_id','=',branch_id])
        if department_id:
            dpt_name=get_model('clinic.department').browse(department_id).name
            dom.append(['name','=',dpt_name])
        dpts=get_model("clinic.department").search_read(dom,['name'])
        dpts=sorted(dpts, key=lambda b: b['name'])
        staff_types=set()
        snames=sorted(staffs.keys()) #sort by staff name
        for sname in snames:
            vals=staffs[sname]
            lvals={
                'first_name': vals.get("first_name") or "",
                'staff_name': sname,
                'staff_id': vals.get('staff_id'),
                'staff_type': vals.get('staff_type'),
                'staff_level': vals.get('staff_level'),
                'categ_name': vals.get('categ_name'),
                'categ_id': vals.get('categ_id'),
                'number': vals.get("number"),
            }
            total=0
            total_qty=0
            lvals['sub_lines']=[]
            for dpt in dpts:
                amt=0 #solve bug
                qty=0
                dname=dpt['name'] or ''
                vals2=vals.get(dname)
                if vals2:
                    amt=vals2.get("amt",0)
                    qty=vals2.get("qty",0)
                    _staff_type=vals2.get("type",'')
                    staff_types.update({_staff_type})
                lvals['sub_lines'].append({'amt': amt,'qty': qty, 'dpt_id': dpt['id']})
                total+=amt
                total_qty+=qty
            lvals['total']=total
            lvals['qty']=qty
            lines.append(lvals)

        title=''
        for stype in list(staff_types):
            if stype=='doctor':
                title+='Doctor'
            elif stype=='nurse':
                title+='Nurse'
        title=' and'.join(t for t in title.split(" "))

        total_lines=[{'amt': 0, 'qty':0} for dpt in dpts]
        for line in lines:
            i=0
            for sub_line in line['sub_lines']:
                amt=sub_line['amt'] or 0
                qty=sub_line['qty'] or 0
                total_lines[i]['amt']+=amt
                total_lines[i]['qty']+=qty
                i+=1
        total=0
        total_qty=0
        total_cost=0
        for tline in total_lines:
            tline['amt']=round(tline['amt'],0) or 0 #XXX
            amt=tline['amt'] or 0
            qty=tline['qty'] or 0
            total+=amt
            total_cost+=amt
            total_qty+=qty

        total_lines.append({'amt': round(total,0)}) #XXX
        company_id=get_active_company()
        comp=get_model("company").browse(company_id)
        if staff_type!='doctor':
            total_hdcase=0
            for k,hdcase_qty in citems.items():
                total_hdcase+=hdcase_qty
        # run no
        nlines=[]
        no=1
        sort_key='staff_name'
        if staff_type=='nurse' and categ_id:
            categ=get_model('clinic.staff.categ').browse(categ_id) 
            if categ.name in ("FT","HP"):
                sort_key='number'
            
        for line in sorted(lines, key=lambda x: (x[sort_key])):
            line['no']=no
            nlines.append(line)
            no+=1

        data={
            'title': title,
            'date_from': date_from,
            'date_to': date_to,
            'dpts': dpts,
            'comp_name': comp.name or 0,
            'comp_span': len(dpts),
            'lines': nlines,
            'total_lines': total_lines,
            'total_hdcase': int2ths(total_hdcase) or 0,
            'total_cost': round(total_cost,0) or 0,
            'staff_type': staff_type,
            'branch_id': branch_id,
            'department_id': department_id,
            }

        # for report xlsx
        data['dpts_txt']=[{0: '#', 1: 'รหัส', 2: 'ชื่อ-สกุล',}]
        items=[]
        vals={}
        no=0
        dpt_len=len(data['dpts_txt'][0])
        for dpt in dpts:
            data['dpts_txt'][0][no+dpt_len]=dpt['name']
            vals[no]=0
            no+=1
        count=1
        for i in range(len(data['lines'])):
            line=data['lines'][i]
            item_vals=vals.copy()
            item_vals['no']=count
            item_vals['number']=line['number']
            item_vals['staff_name']=line['staff_name']
            no=0
            for sub_line in line['sub_lines']:
                item_vals['item%s'%no]=sub_line['amt']
                no+=1
            item_vals['total']=line['total']
            items.append(item_vals)
            count+=1
        data['items']=items
        data['total_lines_txt']=[{}]
        no=0
        for tline in data['total_lines']:
            data['total_lines_txt'][0]['item%s'%no]=tline['amt']
            no+=1

        set_active_user(user_id)
        set_active_company(company_id)
        return data

    def onchange_date(self,context={}):
        data=context['data']
        date=data['date']
        year,month,day=date.split("-")
        weekday, total_day=monthrange(int(year), int(month))
        data['date_from']="%s-%s-01"%(year,month)
        data['date_to']="%s-%s-%s"%(year,month,total_day)
        return data

    def onchange_type(self,context={}):
        data=context['data']
        data['staff_id']=None
        return data

    def onchange_branch(self,context={}):
        data=context['data']
        data['department_id']=None
        return data

    

ReportLaborCostSummary.register()
